var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var _names, _timerMap, _closed, _logLevel, _resolve_filename, _format_map, _adapters, _a, _lg, _b;
import { ConsoleAdapter } from "./consolewriter.js";
import inspect from "./inspect.js";
import { DefaultFormatConfig, Format, Formatted, LoggingTypes, } from "./types.js";
const browser = typeof window !== "undefined";
export const DefaultNativeFunctions = {
    startTimer: () => {
        if (browser && window.performance && window.performance.now) {
            return window.performance.now();
        }
        else {
            return Date.now();
        }
    },
    endTimer: (start) => {
        if (browser && window.performance && window.performance.now) {
            return window.performance.now() - start;
        }
        else {
            return Date.now() - start;
        }
    },
};
const consoleAdapter = new ConsoleAdapter();
const PROJECT_ROOT = typeof process !== "undefined" ? process.cwd() : undefined;
const InitialisedAdapters = Symbol("@hibas123/logging:initialisedAdapters");
export class LoggingInterface {
    constructor(names) {
        _names.set(this, void 0);
        _timerMap.set(this, new Map());
        __classPrivateFieldSet(this, _names, names);
        for (const key in this) {
            if (typeof this[key] === "function") {
                this[key] = this[key].bind(this);
            }
        }
    }
    get names() {
        return [...__classPrivateFieldGet(this, _names)];
    }
    debug(...message) {
        this.message(LoggingTypes.Debug, __classPrivateFieldGet(this, _names), message);
    }
    log(...message) {
        this.message(LoggingTypes.Log, __classPrivateFieldGet(this, _names), message);
    }
    warning(...message) {
        this.message(LoggingTypes.Warn, __classPrivateFieldGet(this, _names), message);
    }
    warn(...message) {
        this.message(LoggingTypes.Warn, __classPrivateFieldGet(this, _names), message);
    }
    error(error, ...message) {
        if (!error)
            error = "Empty ERROR was passed, so no informations available";
        if (typeof error === "string") {
            let e = new Error("This is a fake error, to get a stack trace");
            this.message(LoggingTypes.Error, __classPrivateFieldGet(this, _names), [
                error,
                ...message,
                "\n",
                e.stack,
                "\n",
            ]);
        }
        else {
            this.message(LoggingTypes.Error, __classPrivateFieldGet(this, _names), [error.message, "\n", error.stack, "\n", ...message], getCallerFromExisting(error));
        }
    }
    time(id) {
        if (!id)
            id = Math.floor(Math.random() * 899999 + 100000).toString();
        __classPrivateFieldGet(this, _timerMap).set(id, {
            name: id,
            start: LoggingBase.nativeFunctions.startTimer(),
        });
        return {
            id,
            end: () => this.timeEnd(id),
        };
    }
    timeEnd(id) {
        let timer = __classPrivateFieldGet(this, _timerMap).get(id);
        if (timer) {
            let diff = LoggingBase.nativeFunctions.endTimer(timer.start);
            this.message(LoggingTypes.Debug, __classPrivateFieldGet(this, _names), [
                Format.green(`[${timer.name}]`),
                `->`,
                Format.blue(diff.toFixed(4)),
                "ms",
            ]);
            return diff;
        }
        return -1;
    }
}
_names = new WeakMap(), _timerMap = new WeakMap();
export class LoggingBase extends LoggingInterface {
    constructor(options) {
        super((options === null || options === void 0 ? void 0 : options.name) ? [options.name] : []);
        _closed.set(this, false);
        _logLevel.set(this, LoggingTypes.Debug);
        _resolve_filename.set(this, void 0);
        _format_map.set(this, new DefaultFormatConfig());
        _adapters.set(this, new Set());
        options = Object.assign({ console: true, resolve_filename: true }, options);
        if (options.console) {
            this.addAdapter(consoleAdapter);
        }
    }
    set logLevel(level) {
        __classPrivateFieldSet(this, _logLevel, level);
    }
    get logLevel() {
        return __classPrivateFieldGet(this, _logLevel);
    }
    async addAdapter(adapter) {
        const init = adapter.init();
        const add = () => {
            __classPrivateFieldGet(this, _adapters).add(adapter);
            if (LoggingBase[InitialisedAdapters].has(adapter)) {
                LoggingBase[InitialisedAdapters].set(adapter, LoggingBase[InitialisedAdapters].get(adapter) + 1);
            }
            else {
                LoggingBase[InitialisedAdapters].set(adapter, 1);
            }
        };
        if (!init) {
            add();
        }
        else {
            Promise.resolve(init).then(add);
        }
    }
    getChild(name) {
        return new LoggingBase.DecoupledLogging([...this.names, name], this);
    }
    message(type, names, message, caller) {
        if (__classPrivateFieldGet(this, _logLevel) > type)
            return;
        if (__classPrivateFieldGet(this, _closed)) {
            //TODO: Maybe error?
            return;
        }
        const date = new Date();
        const date_str = date.toISOString().replace(/T/, " ").replace(/\..+/, "");
        let file = undefined;
        if (__classPrivateFieldGet(this, _resolve_filename)) {
            let file_raw = caller;
            if (!file_raw) {
                try {
                    file_raw = getCallerFile();
                }
                catch (err) {
                    file_raw = {
                        file: "<unknown>",
                        line: 0,
                        column: 0,
                    };
                }
            }
            if (PROJECT_ROOT &&
                file_raw.file &&
                file_raw.file.startsWith(PROJECT_ROOT)) {
                let newF = file_raw.file.substring(PROJECT_ROOT.length);
                if (newF.startsWith("/") || newF.startsWith("\\"))
                    newF = newF.substring(1);
                file_raw.file = newF;
            }
            file = `${file_raw.file || "<unknown>"}:${file_raw.line}:${file_raw.column || 0}`;
        }
        let type_str = LoggingTypes[type].toUpperCase().padEnd(5, " ");
        let type_format;
        switch (type) {
            case LoggingTypes.Log:
                type_format = __classPrivateFieldGet(this, _format_map).log;
                break;
            case LoggingTypes.Error:
                type_format = __classPrivateFieldGet(this, _format_map).error;
                break;
            case LoggingTypes.Debug:
                type_format = __classPrivateFieldGet(this, _format_map).debug;
                break;
            case LoggingTypes.Warn:
                type_format = __classPrivateFieldGet(this, _format_map).warning;
                break;
        }
        const nameFormatted = [];
        if (names.length > 0) {
            nameFormatted.push(new Formatted("["));
            for (let i = 0; i < names.length; i++) {
                nameFormatted.push(new Formatted(names[i], __classPrivateFieldGet(this, _format_map).names));
                if (i < names.length - 1) {
                    nameFormatted.push(__classPrivateFieldGet(this, _format_map).names_delimiter);
                }
            }
            nameFormatted.push(new Formatted("]"));
        }
        let linePrefix = [
            new Formatted("["),
            new Formatted(date_str, __classPrivateFieldGet(this, _format_map).date),
            new Formatted("]["),
            new Formatted(type_str, type_format),
            ...(file
                ? [new Formatted("]["), new Formatted(file, __classPrivateFieldGet(this, _format_map).file)]
                : []),
            new Formatted("]"),
            ...nameFormatted,
            new Formatted(": "),
        ];
        let formattedMessage = [...linePrefix];
        message.forEach((msg, idx) => {
            let format = new Formatted();
            if (msg instanceof Formatted) {
                format = msg;
                msg = msg.content;
            }
            if (typeof msg !== "string") {
                msg = inspect(msg, {
                    colors: false,
                    showHidden: true,
                    depth: 3,
                });
            }
            removeColors(msg)
                .split("\n")
                .forEach((text, index, { length }) => {
                if (index != length - 1) {
                    formattedMessage.push(new Formatted(text + "\n", format), ...linePrefix);
                }
                else {
                    formattedMessage.push(new Formatted(text, format));
                }
            });
            formattedMessage.push(new Formatted(" "));
        });
        let msg = {
            date,
            file,
            names,
            text: formattedMessage,
            type,
        };
        __classPrivateFieldGet(this, _adapters).forEach((adapter) => {
            if (adapter.level <= type) {
                adapter.onMessage(msg);
            }
        });
    }
    async close() {
        if (__classPrivateFieldGet(this, _closed))
            return;
        __classPrivateFieldSet(this, _closed, true);
        for (const adapter of __classPrivateFieldGet(this, _adapters)) {
            const cnt = LoggingBase[InitialisedAdapters].get(adapter);
            if (!cnt) {
                //TODO: should not happen!
            }
            else {
                if (cnt <= 1) {
                    if (adapter.close)
                        await adapter.close();
                    LoggingBase[InitialisedAdapters].delete(adapter);
                }
                else {
                    LoggingBase[InitialisedAdapters].set(adapter, cnt - 1);
                }
            }
        }
    }
}
_closed = new WeakMap(), _logLevel = new WeakMap(), _resolve_filename = new WeakMap(), _format_map = new WeakMap(), _adapters = new WeakMap(), _a = InitialisedAdapters;
LoggingBase[_a] = new Map();
LoggingBase.nativeFunctions = DefaultNativeFunctions;
LoggingBase.DecoupledLogging = (_b = class extends LoggingInterface {
        constructor(names, lg) {
            super(names);
            _lg.set(this, void 0);
            __classPrivateFieldSet(this, _lg, lg);
        }
        message(...params) {
            __classPrivateFieldGet(this, _lg).message(...params);
        }
        getChild(name) {
            return new LoggingBase.DecoupledLogging([...this.names, name], __classPrivateFieldGet(this, _lg));
        }
    },
    _lg = new WeakMap(),
    _b);
function getStack() {
    // Save original Error.prepareStackTrace
    let origPrepareStackTrace = Error.prepareStackTrace;
    try {
        // Override with function that just returns `stack`
        Error.prepareStackTrace = function (_, stack) {
            return stack;
        };
        // Create a new `Error`, which automatically gets `stack`
        let err = new Error();
        // Evaluate `err.stack`, which calls our new `Error.prepareStackTrace`
        let stack = err.stack;
        return stack;
    }
    finally {
        // Restore original `Error.prepareStackTrace`
        Error.prepareStackTrace = origPrepareStackTrace;
    }
}
function getCallerFile() {
    try {
        let stack = getStack();
        let current_file = stack.shift().getFileName();
        while (stack.length) {
            let caller_file = stack.shift();
            if (current_file !== caller_file.getFileName()) {
                return {
                    file: caller_file.getFileName(),
                    line: caller_file.getLineNumber(),
                    column: caller_file.getColumnNumber(),
                };
            }
        }
    }
    catch (err) { }
    return { file: undefined, line: 0 };
}
function getCallerFromExisting(err) {
    if (!err || !err.stack)
        return { file: "NOFILE", line: 0 };
    let lines = err.stack.split("\n");
    lines.shift(); // removing first line
    while (lines.length > 0) {
        let line = lines.shift();
        let matches = line.match(/[<]?([a-zA-Z]:)?([\/\\]?[a-zA-Z_-])+[.>][a-zA-Z_-]*([:][0-9]+)+/g);
        if (matches && matches.length > 0) {
            let match = matches[0].trim();
            let locationString = match.match(/([:][0-9]+)+$/gm)[0];
            let line;
            let column;
            if (locationString) {
                match = match.slice(0, match.length - locationString.length);
                locationString = locationString.substring(1);
                [line, column] = locationString.split(":").map(Number);
            }
            let file = match;
            return {
                file,
                line,
                column,
            };
        }
    }
}
export function removeColors(text) {
    text = text.replace(/[\u001b\u009b][[()#;?]*(?:[0-9]{1,4}(?:;[0-9]{0,4})*)?[0-9A-ORZcf-nqry=><]/g, "");
    return text;
}
//# sourceMappingURL=base.js.map