"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConsoleAdapter = void 0;
const index_js_1 = require("./index.js");
const types_js_1 = require("./types.js");
const browser = typeof window !== "undefined" && typeof Deno === "undefined";
const deno = typeof Deno !== "undefined";
const NodeJS = typeof process !== undefined;
class ConsoleAdapter {
    constructor(colors = true) {
        this.colors = colors;
        this.level = types_js_1.LoggingTypes.Debug;
    }
    init() { }
    flush() { }
    setLevel(level) {
        this.level = level;
    }
    // TODO: Check if required!
    // private escape(text: string): string {
    //    return text
    //       .replace(/%s/g, "%%s")
    //       .replace(/%c/g, "%%c")
    // }
    format(formatted) {
        let text = "";
        let style_formats = [];
        if (!browser) {
            // NodeJS or Deno
            for (const format of formatted) {
                let formats = "";
                if (format.bold) {
                    formats += types_js_1.TerminalFormats.Bold;
                }
                if (format.blink) {
                    formats += types_js_1.TerminalFormats.Blink;
                }
                if (format.underscore) {
                    formats += types_js_1.TerminalFormats.Underscore;
                }
                if (format.color) {
                    switch (format.color) {
                        case index_js_1.Colors.RED:
                            formats += types_js_1.TerminalFormats.FgRed;
                            break;
                        case index_js_1.Colors.GREEN:
                            formats += types_js_1.TerminalFormats.FgGreen;
                            break;
                        case index_js_1.Colors.YELLOW:
                            formats += types_js_1.TerminalFormats.FgYellow;
                            break;
                        case index_js_1.Colors.BLUE:
                            formats += types_js_1.TerminalFormats.FgBlue;
                            break;
                        case index_js_1.Colors.MAGENTA:
                            formats += types_js_1.TerminalFormats.FgMagenta;
                            break;
                        case index_js_1.Colors.CYAN:
                            formats += types_js_1.TerminalFormats.FgCyan;
                            break;
                        case index_js_1.Colors.WHITE:
                            formats += types_js_1.TerminalFormats.FgWhite;
                            break;
                    }
                }
                if (format.bgcolor) {
                    switch (format.bgcolor) {
                        case index_js_1.Colors.RED:
                            formats += types_js_1.TerminalFormats.BgRed;
                            break;
                        case index_js_1.Colors.GREEN:
                            formats += types_js_1.TerminalFormats.BgGreen;
                            break;
                        case index_js_1.Colors.YELLOW:
                            formats += types_js_1.TerminalFormats.BgYellow;
                            break;
                        case index_js_1.Colors.BLUE:
                            formats += types_js_1.TerminalFormats.BgBlue;
                            break;
                        case index_js_1.Colors.MAGENTA:
                            formats += types_js_1.TerminalFormats.BgMagenta;
                            break;
                        case index_js_1.Colors.CYAN:
                            formats += types_js_1.TerminalFormats.BgCyan;
                            break;
                        case index_js_1.Colors.WHITE:
                            formats += types_js_1.TerminalFormats.BgWhite;
                            break;
                    }
                }
                text += formats + format.content + types_js_1.TerminalFormats.Reset;
                // (formats.length > 0 ? TerminalFormats.Reset : ""); //TODO: Benchmark if this is better
            }
        }
        else {
            for (const format of formatted) {
                let styles = [];
                let resetStyles = [];
                if (format.bold) {
                    styles.push("font-weight: bold;");
                    resetStyles.push("font-weight: unset");
                }
                if (format.underscore) {
                    styles.push("text-decoration: underline");
                    resetStyles.push("text-decoration: unset");
                }
                if (format.blink) {
                    styles.push("text-decoration: blink");
                    resetStyles.push("text-decoration: unset");
                }
                if (format.color) {
                    let color = "";
                    switch (format.color) {
                        case index_js_1.Colors.RED:
                            color = "red";
                            break;
                        case index_js_1.Colors.GREEN:
                            color = "green";
                            break;
                        case index_js_1.Colors.YELLOW:
                            color = "gold";
                            break;
                        case index_js_1.Colors.BLUE:
                            color = "blue";
                            break;
                        case index_js_1.Colors.MAGENTA:
                            color = "magenta";
                            break;
                        case index_js_1.Colors.CYAN:
                            color = "cyan";
                            break;
                        case index_js_1.Colors.WHITE:
                            color = "white";
                            break;
                    }
                    styles.push("color: " + color);
                    resetStyles.push("color: unset");
                }
                if (format.bgcolor) {
                    let color = "";
                    switch (format.bgcolor) {
                        case index_js_1.Colors.RED:
                            color = "red";
                            break;
                        case index_js_1.Colors.GREEN:
                            color = "green";
                            break;
                        case index_js_1.Colors.YELLOW:
                            color = "gold";
                            break;
                        case index_js_1.Colors.BLUE:
                            color = "blue";
                            break;
                        case index_js_1.Colors.MAGENTA:
                            color = "magenta";
                            break;
                        case index_js_1.Colors.CYAN:
                            color = "cyan";
                            break;
                        case index_js_1.Colors.WHITE:
                            color = "white";
                            break;
                    }
                    styles.push("background-color: " + color);
                    resetStyles.push("background-color: unset");
                }
                text += "%c" + format.content.replace(/%c/g, "%%c") + "%c";
                style_formats.push(styles.join(";"), resetStyles.join(";"));
            }
        }
        return [text, style_formats];
    }
    onMessage(message) {
        if (browser) {
            if (this.colors) {
                const [text, formats] = this.format(message.text);
                console.log(text, ...formats);
            }
            else {
                console.log(types_js_1.Formatted.strip(message.text));
            }
        }
        else {
            const text = this.colors
                ? this.format(message.text)[0]
                : types_js_1.Formatted.strip(message.text);
            if (deno) {
                //TODO: Deno specific thing
                console.log(text);
            }
            else if (typeof process !== "undefined") {
                //NodeJS
                process.stdout.write(text + "\n");
            }
            else {
                console.log(text);
            }
        }
    }
}
exports.ConsoleAdapter = ConsoleAdapter;
//# sourceMappingURL=consolewriter.js.map